@echo off
  rem qhasm.bat [-debug] [-amd64|-x86] [-c] file.[p]q [-o file.s]
  rem -----------------------------------------------------------
  rem examples:
  rem   qhasm test.q
  rem   qhasm -debug -x86 test.pq -o test.s
  rem   qhasm -c test.pq -o test.obj
  rem -----------------------------------------------------------
  rem windows native shell wrapper for
  rem   D.J.Bernstein and Peter Schwabe qhasm / maq tools
  rem -----------------------------------------------------------
  rem pre-requisites:
  rem   sed, gawk, .\qhast-ops.exe, .\qhasm-regs.exe
  rem   .\maq.bat (python, MinGW gcc, tr, grep, .\*.py)
  rem   .\AMD64.processor, .\X64.processor
  rem   .\AMD64.sed, .\x64.sed
  rem   .\qhasm-as.awk, .\qhasm-fp.awk
  rem   .\settings.bat
  rem -----------------------------------------------------------

  set QHDIR=%~dp0
  set QHDIR=%QHDIR:~0,-1%
  set DBG=N& set DBS=& set PA=AMD64& set S=& set O=& set SO=& set OO=& set COMPILE=N& set PQ=
:param
  if "" == "%1" goto :cont
  for %%p in (DBG DEB DEBUG) do if /i "%1" == "-%%p" set DBG=Y& set DBS= -debug&       shift& goto :param
  for %%p in (X86 AMD64 SPARC PPC-LINUX PPC-MACOS) do if /i "%1" == "-%%p" set PA=%%p& shift& goto :param
  if "-c" == "%1" set COMPILE=Y&                                                       shift& goto :param
  if "-o" == "%1" set SO=%~2&                                                   shift& shift& goto :param
  if not "" == "%S%" goto :usage
  set S=%~1& set O=%~n1
  if not exist "%S%" goto :usage
  shift& goto :param
:cont
  if "" == "%SO%" set SO=%O%.s
  if ".o"   == "%SO:~-2%" set COMPILE=Y& set OO=%SO%& set SO=%SO:~0,-2%.s
  if ".obj" == "%SO:~-4%" set COMPILE=Y& set OO=%SO%& set SO=%SO:~0,-4%.s
  if "Y" == "%COMPILE%" if "" == "%OO%" set OO=%O%.obj
  rem echo O=%O% SO=%SO% OO=%OO% PA=%PA% DBG=%DBG% S=%1 COMPILE=%COMPILE%
  if not "" == "%S%" goto :work
:usage
  echo.
  echo Usage:
  echo   qhasm.bat [-debug] [-amd64^|-x86] file.(pq^|q^) [-o file.s]
  echo   qhasm.bat [-debug] [-amd64^|-x86] -c file.(pq^|q^|s) [-o file.obj]
  echo.
  echo Examples:
  echo   qhasm test.q
  echo   qhasm -debug -x86 test.pq -o test.s
  echo   qhasm -amd64 test.pq
  echo   qhasm -c test.pq -o test.obj
goto :EOF

:work
  rem exit
  call "%QHDIR%\settings.bat" 

  rem do preprocessing if source file extension is .pq
  rem 
  if /i "%S:~-3%" == ".pq" (
    call "%QHDIR%\maq.bat"%DBS% %S% %O%.q
    set PQ=Y
    set S=%O%.q
  )
  if /i "%S:~-2%" == ".s" goto :asm_compile
  echo qhasm%DBS% -%PA% %S%:

  echo   sed1
  type %S% 2>NUL | %SED% -e "s/#.*//" -e "s/:/#/g" > %O%.a.sed1

  echo   ops
  copy /b "%QHDIR%\%PA%.processor" + %O%.a.sed1 %O%.b.copy 1>NUL
  "%QHDIR%\qhasm-ops.exe" < %O%.b.copy > %O%.c.ops

  echo   regs
  "%QHDIR%\qhasm-regs.exe" < %O%.c.ops > %O%.d.regs

  set op=%PA:-=_%
  call :do_%op%

  if /i not "Y" == "%DBG%" for %%f in (
    %O%.a.sed1 %O%.b.copy %O%.c.ops %O%.d.regs %O%.e.fpas
  ) do if exist %%f del /q %%f 1>NUL

  if /i not "Y" == "%DBG%" if "Y" == "%PQ%" if exist %O%.q del /q %O%.q 1>NUL
  if "Y" == "%COMPILE%" goto :asm_compile
goto :EOF

:asm_compile
  echo asm_compile %SO%:
  set ASM=
  if "AMD64" == "%PA%"  set ASM=%AS_AMD64%
  if "X86"   == "%PA%"  set ASM=%AS_X86%
  if "SPARC" == "%PA%"  set ASM=%AS_SPARC%
  if ""      == "%ASM%" echo "ASM is undefined for %PA% -- correct 'settings.bat'"& goto :EOF
  %ASM% %SO% -o %OO%  
  if /i not "Y" == "%DBG%" if /i not ".s" == "%S:~-2%" if exist %SO% del /q %SO% 1>NUL
goto :EOF

:do_SPARC
  rem
:do_PPC_LINUX
  echo as
  %GREP% -v "^op:livefloat80:" < %O%.d.regs | %AWK% -F: -f "%QHDIR%\qhasm-as.awk" > %SO%
goto :EOF

:do_PPC_MACOS
  echo as
  %GREP% -v "^op:livefloat80:" < %O%.d.regs | %AWK% -F: -f "%QHDIR%\qhasm-as.awk" | %SED% "s/^# /; /" > %SO%
goto :EOF

:do_X86
  rem
:do_AMD64
  echo   fp-as
  %AWK% -F: -f "%QHDIR%\qhasm-fp.awk" < %O%.d.regs | %AWK% -F: -f "%QHDIR%\qhasm-as.awk" > %O%.e.fpas
  echo   sed2
  %SED% -f "%QHDIR%\%PA%.sed" < %O%.e.fpas > %SO%
goto :EOF
